@@ -11,13 +11,15 @@ use std::{
     time::Instant,
 };
 
-use log::trace;
+use pnet_packet::{icmp, icmpv6, ipv4, ipv6, Packet};
 use socket2::{Domain, Protocol, Socket, Type};
 use tokio::{
     net::UdpSocket,
-    sync::{broadcast, mpsc, Mutex},
+    sync::{mpsc, Mutex},
     task,
 };
+use tracing::warn;
+use uuid::Uuid;
 
 use crate::{config::Config, Pinger, ICMP};
 
@@ -86,59 +88,69 @@ impl AsyncSocket {
 #[derive(Clone)]
 pub struct Client {
     socket: AsyncSocket,
-    mapping: Arc<Mutex<HashMap<IpAddr, mpsc::Sender<Message>>>>,
+    mapping: Arc<Mutex<HashMap<Uuid, mpsc::Sender<Message>>>>,
 }
 
 impl Client {
     /// A client is generated according to the configuration. In fact, a `AsyncSocket` is wrapped inside,
     /// and you can clone to any `task` at will.
-    pub fn new(config: &Config) -> io::Result<Self> {
+    pub async fn new(config: &Config) -> io::Result<Self> {
         let socket = AsyncSocket::new(config)?;
-        Ok(Self {
-            socket,
-            mapping: Arc::new(Mutex::new(HashMap::new())),
-        })
+        let mapping = Arc::new(Mutex::new(HashMap::new()));
+        task::spawn(recv_task(socket.clone(), mapping.clone()));
+        Ok(Self { socket, mapping })
     }
 
     /// Create a `Pinger` instance, you can make special configuration for this instance. Such as `timeout`, `size` etc.
     pub async fn pinger(&self, host: IpAddr) -> Pinger {
-        let (shutdown_tx, _) = broadcast::channel(1);
         let (tx, rx) = mpsc::channel(10);
+        let key = Uuid::new_v4();
         {
-            self.mapping.lock().await.insert(host, tx);
+            self.mapping.lock().await.insert(key, tx);
         }
-        task::spawn(recv_task(
-            self.socket.clone(),
-            self.mapping.clone(),
-            shutdown_tx.subscribe(),
-        ));
-        Pinger::new(host, self.socket.clone(), rx, shutdown_tx)
+        Pinger::new(host, self.socket.clone(), rx, key)
     }
 }
 
-async fn recv_task(
-    socket: AsyncSocket,
-    mapping: Arc<Mutex<HashMap<IpAddr, mpsc::Sender<Message>>>>,
-    mut shutdown_rx: broadcast::Receiver<()>,
-) {
+async fn recv_task(socket: AsyncSocket, mapping: Arc<Mutex<HashMap<Uuid, mpsc::Sender<Message>>>>) {
     let mut buf = [0; 2048];
     loop {
-        tokio::select! {
-            answer = socket.recv_from(&mut buf) => {
-                if let Ok((sz, addr)) = answer {
-                    let instant = Instant::now();
-                    let mut w = mapping.lock().await;
-                    if let Some(tx) = (*w).get(&addr.ip()) {
-                        if tx.send(Message::new(instant, buf[0..sz].to_vec())).await.is_err() {
-                            trace!("send message error");
-                            (*w).remove(&addr.ip());
-                        }
+        if let Ok((sz, addr)) = socket.recv_from(&mut buf).await {
+            let datas = buf[0..sz].to_vec();
+            if let Some(uuid) = gen_uuid_with_payload(addr.ip(), datas.as_slice()) {
+                let instant = Instant::now();
+                let mut w = mapping.lock().await;
+                if let Some(tx) = (*w).get(&uuid) {
+                    if tx.send(Message::new(instant, datas)).await.is_err() {
+                        warn!("Pinger({}) already closed.", addr);
+                        (*w).remove(&uuid);
                     }
                 }
             }
-            _ = shutdown_rx.recv() => {
-                break
-            }
         }
     }
 }
+
+fn gen_uuid_with_payload(addr: IpAddr, datas: &[u8]) -> Option<Uuid> {
+    match addr {
+        IpAddr::V4(_) => {
+            if let Some(ip_packet) = ipv4::Ipv4Packet::new(datas) {
+                if let Some(icmp_packet) = icmp::IcmpPacket::new(ip_packet.payload()) {
+                    let payload = icmp_packet.payload();
+                    let uuid = &payload[4..20];
+                    return Uuid::from_slice(uuid).ok();
+                }
+            }
+        }
+        IpAddr::V6(_) => {
+            if let Some(ipv6_packet) = ipv6::Ipv6Packet::new(datas) {
+                if let Some(icmpv6_packet) = icmpv6::Icmpv6Packet::new(ipv6_packet.payload()) {
+                    let payload = icmpv6_packet.payload();
+                    let uuid = &payload[4..20];
+                    return Uuid::from_slice(uuid).ok();
+                }
+            }
+        }
+    }
+    None
+}
