@@ -1,7 +1,6 @@
 #![allow(dead_code)]
 use std::io;
 
-use pnet_packet::{icmp::IcmpType, icmpv6::Icmpv6Type};
 use thiserror::Error;
 
 pub type Result<T> = std::result::Result<T, SurgeError>;
@@ -16,22 +15,22 @@ pub enum SurgeError {
     MalformedPacket(#[from] MalformedPacketError),
     #[error("io error")]
     IOError(#[from] io::Error),
-    #[error("expected echoreply, got {0:?}")]
-    NotEchoReply(IcmpType),
-    #[error("expected echoreply, got {0:?}")]
-    NotV6EchoReply(Icmpv6Type),
-    #[error("timeout error")]
-    Timeout,
-    #[error("other icmp message")]
-    OtherICMP,
+    #[error("Request timeout for icmp_seq {seq}")]
+    Timeout { seq: u16 },
+    #[error("Echo Request packet.")]
+    EchoRequestPacket,
+    #[error("Network error.")]
+    NetworkError,
 }
 
 #[derive(Error, Debug)]
 pub enum MalformedPacketError {
     #[error("expected an Ipv4Packet")]
     NotIpv4Packet,
-    #[error("expected an IcmpPacket payload")]
-    NotIcmpPacket,
+    #[error("expected an Ipv6Packet")]
+    NotIpv6Packet,
+    #[error("expected an Icmpv4Packet payload")]
+    NotIcmpv4Packet,
     #[error("expected an Icmpv6Packet")]
     NotIcmpv6Packet,
     #[error("payload too short, got {got}, want {want}")]
